package com.company.dndsample.screen.todo;

import com.vaadin.event.dd.DragAndDropEvent;
import com.vaadin.event.dd.DropHandler;
import com.vaadin.event.dd.acceptcriteria.AcceptAll;
import com.vaadin.event.dd.acceptcriteria.AcceptCriterion;
import com.vaadin.shared.ui.dd.VerticalDropLocation;
import com.vaadin.ui.AbstractOrderedLayout;
import com.vaadin.ui.Component;
import io.jmix.ui.UiComponents;
import io.jmix.ui.component.*;
import io.jmix.ui.component.Component.Alignment;
import io.jmix.ui.icon.JmixIcon;
import io.jmix.ui.screen.Screen;
import io.jmix.ui.screen.Subscribe;
import io.jmix.ui.screen.UiController;
import io.jmix.ui.screen.UiDescriptor;
import io.jmix.ui.widget.*;
import io.jmix.ui.widget.addon.dragdroplayouts.DDVerticalLayout;
import io.jmix.ui.widget.addon.dragdroplayouts.events.LayoutBoundTransferable;
import io.jmix.ui.widget.client.addon.dragdroplayouts.ui.LayoutDragMode;
import org.springframework.beans.factory.annotation.Autowired;

@UiController("dnd_DDTodoScreen")
@UiDescriptor("ddtodo-screen.xml")
public class DDTodoScreen extends Screen {

    @Autowired
    private UiComponents uiComponents;

    @Autowired
    private GroupBoxLayout rootPalette;
    @Autowired
    private ScrollBoxLayout dashboardScrollBox;

    @Subscribe
    public void onInit(InitEvent event) {
        initPalette(rootPalette);
        initDashboard(dashboardScrollBox);
    }

    private void initPalette(GroupBoxLayout rootPalette) {
        DDVerticalLayout ddPalette = new DDVerticalLayout();
        ddPalette.setDragMode(LayoutDragMode.CLONE);
        ddPalette.addComponent(createPaletteButton("callBtn", "Call"));
        ddPalette.addComponent(createPaletteButton("chatBtn", "Chat"));
        ddPalette.addComponent(createPaletteButton("meetingBtn", "Meeting"));
        ddPalette.addComponent(createPaletteButton("buyBtn", "Buy"));

        rootPalette.withUnwrapped(JmixGroupBox.class, vGroupBox -> vGroupBox.addComponent(ddPalette));
    }

    private JmixButton createPaletteButton(String id, String caption) {
        Button callBtn = uiComponents.create(Button.NAME);
        callBtn.setId(id);
        callBtn.setCaption(caption);
        callBtn.setWidthFull();
        return callBtn.unwrap(JmixButton.class);
    }

    private void initDashboard(ScrollBoxLayout scrollBox) {
        DDVerticalLayout dashboard = new DDVerticalLayout();
        dashboard.setDragMode(LayoutDragMode.CLONE);
        dashboard.setDropHandler(new DropHandler() {
            @Override
            public void drop(DragAndDropEvent event) {
                onDrop(event);
            }

            @Override
            public AcceptCriterion getAcceptCriterion() {
                return AcceptAll.get();
            }
        });

        HtmlAttributesExtension.get(dashboard).setCssProperty("min-height", "100px");

        scrollBox.withUnwrapped(JmixScrollBoxLayout.class, vScrollBox-> vScrollBox.addComponent(dashboard));
    }

    protected void onDrop(DragAndDropEvent event) {
        LayoutBoundTransferable t = (LayoutBoundTransferable) event.getTransferable();
        DDVerticalLayout.VerticalLayoutTargetDetails details = (DDVerticalLayout.VerticalLayoutTargetDetails) event.getTargetDetails();

        Component sourceLayout = t.getSourceComponent();
        DDVerticalLayout targetLayout = (DDVerticalLayout) details.getTarget();
        Component tComponent = t.getComponent();

        VerticalDropLocation loc = details.getDropLocation();

        int indexTo = details.getOverIndex();
        int indexFrom = targetLayout.getComponentIndex(tComponent);

        if (tComponent == null) {
            return;
        }
        Component componentToAdd;

        if (sourceLayout == targetLayout) {
            componentToAdd = tComponent;
            if (indexFrom == indexTo) {
                return;
            }
            targetLayout.removeComponent(tComponent);
            if (indexTo > indexFrom) {
                indexTo--;
            }
            if (indexTo == -1) {
                targetLayout.addComponent(tComponent, indexFrom);
            }
        } else {
            componentToAdd = createDashboardElement(tComponent);
            if (indexTo == -1) {
                targetLayout.addComponent(componentToAdd, targetLayout.getComponentCount());
            }
        }

        if (indexTo != -1) {
            if (loc == VerticalDropLocation.MIDDLE || loc == VerticalDropLocation.BOTTOM) {
                indexTo++;
            }
            targetLayout.addComponent(componentToAdd, indexTo);
        }
        updateDashboardComponents(targetLayout);
    }

    private Component createDashboardElement(Component tComponent) {
        GroupBoxLayout groupBox = uiComponents.create(GroupBoxLayout.class);
        groupBox.setWidthFull();

        HBoxLayout layout = uiComponents.create(HBoxLayout.class);
        layout.setWidthFull();
        layout.setSpacing(true);

        Label<String> countLabel = uiComponents.create(Label.NAME);
        countLabel.setId("countLabel");
        countLabel.setWidth("30px");
        countLabel.setAlignment(Alignment.MIDDLE_CENTER);

        Label<String> titleLabel = uiComponents.create(Label.NAME);
        titleLabel.setValue(tComponent.getCaption());
        titleLabel.setWidth("60px");
        titleLabel.setAlignment(Alignment.MIDDLE_CENTER);

        TextField<String> textField = uiComponents.create(TextField.NAME);

        Button deleteButton = uiComponents.create(Button.class);
        deleteButton.setIconFromSet(JmixIcon.TIMES);
        deleteButton.addClickListener(event -> {
            DDVerticalLayout ddLayout = (DDVerticalLayout) groupBox.unwrap(JmixGroupBox.class).getParent();
            ddLayout.removeComponent(groupBox.unwrap(JmixGroupBox.class));
            updateDashboardComponents(ddLayout);
        });

        layout.add(countLabel, titleLabel, textField, deleteButton);
        layout.expand(textField);

        groupBox.add(layout);

        return groupBox.unwrap(JmixGroupBox.class);
    }

    private void updateDashboardComponents(DDVerticalLayout layout) {
        for (int i = 0; i < layout.getComponentCount(); i++) {
            JmixGroupBox groupBox = (JmixGroupBox) layout.getComponent(i);
            AbstractOrderedLayout orderedLayout = (AbstractOrderedLayout) groupBox.getComponentIterator().next();
            ((JmixLabel) orderedLayout.getComponent(0)).setValue(String.valueOf(i + 1));
        }
    }
}